@@ -15,20 +15,17 @@ function(Symbol,from='1970-01-01',to=Sys.Date(),env=parent.frame(),src='yahoo',
   from.posix <- .dateToUNIX(from)
   to.posix <- .dateToUNIX(to)
 
-  tmp <- tempfile()
-  on.exit(unlink(tmp))
-
   handle <- .getHandle()
   yahoo.URL <- .yahooURL(Symbol.name, from.posix, to.posix,
                          "1d", "split", handle)
-  curl::curl_download(yahoo.URL, destfile=tmp, quiet=!verbose, handle=handle$ch)
 
-  fr <- read.csv(tmp, as.is=TRUE)
+  fr <- read.csv(curl::curl(yahoo.URL, handle=handle$ch), as.is=TRUE)
 
   if(NROW(fr)==0) {
     fr <- NA
   } else {
-    fr$V3 <- 1 / sapply(parse(text=fr[,2]), eval)
+    fr[,2] <- gsub(":", "/", fr[,2], fixed = TRUE)
+    fr$V3 <- 1 / vapply(parse(text=fr[,2]), eval, numeric(1))
     fr <- xts(fr$V3, as.Date(fr[,1], "%Y-%m-%d"))
     colnames(fr) <- paste(Symbol.name,'spl',sep='.')
   }
