@@ -18,7 +18,7 @@ function(Symbols=NULL,
                 'and getOption("getSymbols.auto.assign") will still be checked for\n',
                 'alternate defaults.\n\n',
                 'This message is shown once per session and may be disabled by setting \n',
-                'options("getSymbols.warning4.0"=FALSE). See ?getSymbols for details.')
+                'options("getSymbols.warning4.0"=FALSE). See ?getSymbols for details.\n')
         options("getSymbols.warning4.0"=FALSE)
       }
       importDefaults("getSymbols")
@@ -81,6 +81,8 @@ function(Symbols=NULL,
                              Symbols[Symbols==x]
                            }
                            )))
+        # was getSymbols() called with more than 1 symbol?
+        .has1sym. <- length(Symbols) < 2L
         #Symbols <- as.list(Symbols)
         all.symbols <- list()
         for(symbol.source in unique(as.character(Symbols))) {
@@ -91,7 +93,8 @@ function(Symbols=NULL,
                                            #reload.Symbols=reload.Symbols,
                                            verbose=verbose,warnings=warnings,
                                            auto.assign=auto.assign,
-                                           ...))
+                                           ...,
+                                           .has1sym.=.has1sym.))
           if(!auto.assign)
             return(symbols.returned)
           for(each.symbol in symbols.returned) all.symbols[[each.symbol]] <- symbol.source 
@@ -222,9 +225,6 @@ formals(loadSymbols) <- loadSymbols.formals
 
     # establish session
     new.session <- function() {
-      tmp <- tempfile()
-      on.exit(unlink(tmp))
-
       for (i in 1:5) {
         h <- curl::new_handle()
         curl::handle_setopt(h, .list = curl.options)
@@ -232,7 +232,7 @@ formals(loadSymbols) <- loadSymbols.formals
         # random query to avoid cache
         ru <- paste(sample(c(letters, 0:9), 4), collapse = "")
         cu <- paste0("https://finance.yahoo.com?", ru)
-        curl::curl_download(cu, tmp, handle = h)
+        z <- curl::curl_fetch_memory(cu, handle = h)
         if (NROW(curl::handle_cookies(h)) > 0)
           break;
         Sys.sleep(0.1)
@@ -264,8 +264,8 @@ function(symbol, from, to, period, type, handle)
   e <- match.arg(type, c("history", "div", "split"))
   n <- if (unclass(Sys.time()) %% 1L >= 0.5) 1L else 2L
   u <- paste0("https://query", n, ".finance.yahoo.com/v7/finance/download/",
-              symbol, "?period1=", from, "&period2=", to, "&interval=", p,
-              "&events=", e, "&crumb=", handle$cb)
+              symbol, sprintf("?period1=%.0f&period2=%.0f", from, to),
+              "&interval=", p, "&events=", e, "&crumb=", handle$cb)
   return(u)
 }
 
@@ -283,15 +283,6 @@ function(Symbols,env,return.class='xts',index.class="Date",
          periodicity="daily",
          curl.options=list())
 {
-     if(getOption("getSymbols.yahoo.warning",TRUE)) {
-       # Warn about Yahoo Finance quality and stability
-       message("\nWARNING: There have been significant changes to Yahoo Finance data.",
-               "\nPlease see the Warning section of ", sQuote("?getSymbols.yahoo"), " for details.\n",
-               "\nThis message is shown once per session and may be disabled by setting\n",
-               "options(\"getSymbols.yahoo.warning\"=FALSE).")
-       options("getSymbols.yahoo.warning"=FALSE)
-     }
-
      importDefaults("getSymbols.yahoo")
      this.env <- environment()
      for(var in names(list(...))) {
@@ -313,10 +304,10 @@ function(Symbols,env,return.class='xts',index.class="Date",
 
      handle <- .getHandle(curl.options)
 
-     tmp <- tempfile()
-     on.exit(unlink(tmp))
-
-     for(i in 1:length(Symbols)) {
+     returnSym <- Symbols
+     noDataSym <- NULL
+     for(i in seq_along(Symbols)) {
+       test <- try({
        return.class <- getSymbolLookup()[[Symbols[[i]]]]$return.class
        return.class <- ifelse(is.null(return.class),default.return.class,
                               return.class)
@@ -343,30 +334,27 @@ function(Symbols,env,return.class='xts',index.class="Date",
 
        yahoo.URL <- .yahooURL(Symbols.name, from.posix, to.posix,
                               interval, "history", handle)
-       dl <- try(curl::curl_download(yahoo.URL, destfile = tmp,
-                                     quiet = !verbose, handle = handle$ch),
-                  silent = TRUE)
+       fr <- try(read.csv(curl::curl(yahoo.URL, handle = handle$ch), na.strings="null"),
+                 silent = TRUE)
 
-       if (inherits(dl, "try-error")) {
+       if (inherits(fr, "try-error")) {
          # warn user about the failure
          warning(Symbols.name, " download failed; trying again.",
                  call. = FALSE, immediate. = TRUE)
          # re-create handle
          handle <- .getHandle(curl.options, force.new = TRUE)
-         # try again
+         # try again. must rebuild url with crumbs
          yahoo.URL <- .yahooURL(Symbols.name, from.posix, to.posix,
                                 interval, "history", handle)
-         dl <- try(curl::curl_download(yahoo.URL, destfile = tmp,
-                                       quiet = !verbose, handle = handle$ch),
-                    silent = TRUE)
+         fr <- try(read.csv(curl::curl(yahoo.URL, handle = handle$ch), na.strings="null"),
+                   silent = TRUE)
          # error if second attempt also failed
-         if (inherits(dl, "try-error")) {
+         if (inherits(fr, "try-error")) {
            stop(Symbols.name, " download failed after two attempts. Error",
-                " message:\n", attr(dl, "condition")$message, call. = FALSE)
+                " message:\n", attr(fr, "condition")$message, call. = FALSE)
          }
        }
 
-       fr <- read.csv(tmp, na.strings="null")
        if(verbose) cat("done.\n")
        fr <- xts(as.matrix(fr[,-1]),
                  as.Date(fr[,1]),
@@ -394,7 +382,7 @@ function(Symbols,env,return.class='xts',index.class="Date",
 
        fr <- convert.time.series(fr=fr,return.class=return.class)
        if(is.xts(fr))
-         indexClass(fr) <- index.class
+         tclass(fr) <- index.class
 
        Symbols[[i]] <-toupper(gsub('\\^','',Symbols[[i]])) 
        if(auto.assign)
@@ -403,9 +391,19 @@ function(Symbols,env,return.class='xts',index.class="Date",
          message("pausing 1 second between requests for more than 5 symbols")
          Sys.sleep(1)
        }
+       }, silent = TRUE)
+       if (inherits(test, "try-error")) {
+         msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                       ".\n", attr(test, "condition")$message)
+         if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+           stop(msg)
+         }
+         warning(msg, call. = FALSE, immediate. = TRUE)
+         noDataSym <- c(noDataSym, returnSym[[i]])
+       }
      }
      if(auto.assign)
-       return(Symbols)
+       return(setdiff(returnSym, noDataSym))
      return(fr)
 }
 # }}}
@@ -437,7 +435,11 @@ function(Symbols,env,return.class='xts',index.class="Date",
           stop("package:",dQuote("XML"),"cannot be loaded.")
 
         yahoo.URL <- "https://info.finance.yahoo.co.jp/history/"
-        for(i in 1:length(Symbols)) {
+
+        returnSym <- Symbols
+        noDataSym <- NULL
+        for(i in seq_along(Symbols)) {
+            test <- try({
             # The name of the symbol, which will actually be used as the
             # variable name. It needs to start with YJ, and it will be appended
             # if it does not.
@@ -481,10 +483,8 @@ function(Symbols,env,return.class='xts',index.class="Date",
             
             page <- 1
             totalrows <- c()
-            tmp <- tempfile()
-            on.exit(unlink(tmp))
             while (TRUE) {
-                download.file(paste(yahoo.URL,
+                URL <- paste(yahoo.URL,
                                     "?code=",Symbols.name,
                                     "&sm=",from.m,
                                     "&sd=",sprintf('%.2d',from.d),
@@ -494,17 +494,21 @@ function(Symbols,env,return.class='xts',index.class="Date",
                                     "&ey=",to.y,
                                     "&tm=d",
                                     "&p=",page,
-                                    sep=''),destfile=tmp,quiet=!verbose)
+                                    sep='')
                 
-                fdoc <- XML::htmlParse(tmp)
+                fdoc <- XML::htmlParse(URL)
                 rows <- XML::xpathApply(fdoc, "//table[@class='boardFin yjSt marB6']//tr")
-                if (length(rows) == 1) break
+                if (length(rows) <= 1) break
                 
                 totalrows <- c(totalrows, rows)
                 page <- page + 1
             }
             if(verbose) cat("done.\n")
             
+            if (is.null(rows)) {
+              stop("No historical data for ", dQuote(Symbols[[i]]), ".")
+            }
+
             # Available columns
             cols <- c('Open','High','Low','Close','Volume','Adjusted')
             
@@ -520,7 +524,7 @@ function(Symbols,env,return.class='xts',index.class="Date",
                 
                 # 2 cells means it is a stocksplit row
                 # So extract stocksplit data and recalculate the matrix we have so far
-                if (length(cells) == 2 & length(cols) == 6 & nrow(mat) > 1) {
+                if (length(cells) == 2 && length(cols) == 6 & nrow(mat) > 1) {
                     ss.data <- as.numeric(na.omit(as.numeric(unlist(strsplit(XML::xmlValue(cells[[2]]), "[^0-9]+")))))
                     factor <- ss.data[2] / ss.data[1]
                     
@@ -550,7 +554,7 @@ function(Symbols,env,return.class='xts',index.class="Date",
             
             fr <- convert.time.series(fr=fr,return.class=return.class)
             if(is.xts(fr))
-                indexClass(fr) <- index.class
+                tclass(fr) <- index.class
             
             Symbols[[i]] <- symname
             if(auto.assign)
@@ -560,9 +564,19 @@ function(Symbols,env,return.class='xts',index.class="Date",
                 Sys.sleep(1)
             }
             
+            }, silent = TRUE)
+            if (inherits(test, "try-error")) {
+                msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                              ".\n", attr(test, "condition")$message)
+                if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+                  stop(msg)
+                }
+                warning(msg, call. = FALSE, immediate. = TRUE)
+                noDataSym <- c(noDataSym, returnSym[[i]])
+            }
         }
         if(auto.assign)
-            return(Symbols)
+            return(setdiff(returnSym, noDataSym))
         return(fr)
     }
 # }}}
@@ -574,88 +588,11 @@ function(Symbols,env,return.class='xts',
          to=Sys.Date(),
          ...)
 {
-     fix.google.bug <- TRUE
-     importDefaults("getSymbols.google")
-     this.env <- environment()
-     for(var in names(list(...))) {
-        # import all named elements that are NON formals
-        assign(var, list(...)[[var]], this.env)
-     }
-
-     default.return.class <- return.class
-     default.from <- from
-     default.to <- to
-
-     if(!hasArg("verbose")) verbose <- FALSE
-     if(!hasArg("auto.assign")) auto.assign <- TRUE
-     google.URL <- "http://finance.google.com/finance/historical?"
-
-     # Google CSV contains English month abbreviations
-     # Ensure strptime() uses an English locale in this call
-     lc_time <- Sys.getlocale("LC_TIME")
-     on.exit(Sys.setlocale(category = "LC_TIME", locale = lc_time))
-     Sys.setlocale(category = "LC_TIME", locale = "C")
-
-     tmp <- tempfile()
-     on.exit(unlink(tmp), add = TRUE)
-
-     for(i in 1:length(Symbols)) {
-       return.class <- getSymbolLookup()[[Symbols[[i]]]]$return.class
-       return.class <- ifelse(is.null(return.class),default.return.class,
-                              return.class)
-       from <- getSymbolLookup()[[Symbols[[i]]]]$from
-       from <- if(is.null(from)) default.from else from
-       to <- getSymbolLookup()[[Symbols[[i]]]]$to
-       to <- if(is.null(to)) default.to else to
-
-       from.y <- as.numeric(strsplit(as.character(from),'-',)[[1]][1])
-       from.m <- as.numeric(strsplit(as.character(from),'-',)[[1]][2])
-       from.d <- as.numeric(strsplit(as.character(from),'-',)[[1]][3])
-       to.y <- as.numeric(strsplit(as.character(to),'-',)[[1]][1])
-       to.m <- as.numeric(strsplit(as.character(to),'-',)[[1]][2])
-       to.d <- as.numeric(strsplit(as.character(to),'-',)[[1]][3])
-
-       Symbols.name <- getSymbolLookup()[[Symbols[[i]]]]$name
-       Symbols.name <- ifelse(is.null(Symbols.name),Symbols[[i]],Symbols.name)
-       if(verbose) cat("downloading ",Symbols.name,".....\n\n")
-       download.file(paste(google.URL,
-                           "q=",Symbols.name,
-                           "&startdate=",month.abb[from.m],
-                           "+",sprintf('%.2d',from.d),
-                           ",+",from.y,
-                           "&enddate=",month.abb[to.m],
-                           "+",sprintf('%.2d',to.d),
-                           ",+",to.y,
-                           "&output=csv",
-                           sep=''),destfile=tmp,quiet=!verbose)
-       fr <- read.csv(tmp)
-       if(verbose) cat("done.\n")
-       fr <- fr[nrow(fr):1,] #google data is backwards
-       if(fix.google.bug) {
-         bad.dates <- c('29-Dec-04','30-Dec-04','31-Dec-04')
-         if(as.Date(from,origin='1970-01-01') < as.Date("2003-12-28",origin='1970-01-01') &&
-            as.Date(to,origin='1970-01-01') > as.Date("2003-12-30",origin='1970-01-01')) {
-           dup.dates <- which(fr[,1] %in% bad.dates)[(1:3)]
-           fr <- fr[-dup.dates,]
-           warning("google duplicate bug - missing Dec 28,29,30 of 2003")
-         }
-       }
-       fr <- xts(as.matrix(fr[,-1]),
-                 as.Date(strptime(fr[,1],"%d-%B-%y"),origin='1970-01-01'),
-                 src='google',updated=Sys.time())
-       colnames(fr) <- paste(toupper(gsub('\\^','',Symbols.name)),
-                             c('Open','High','Low','Close','Volume'),
-                             sep='.')
-       # convert '-' to NAs
-       suppressWarnings(storage.mode(fr) <- "numeric")
-       fr <- convert.time.series(fr=fr,return.class=return.class)
-       Symbols[[i]] <-toupper(gsub('\\^','',Symbols[[i]])) 
-       if(auto.assign)
-         assign(Symbols[[i]],fr,env)
-     }
-     if(auto.assign)
-       return(Symbols)
-     return(fr)
+  msg <- paste0(sQuote("getSymbols.google"), " is defunct.",
+         "\nGoogle Finance stopped providing data in March, 2018.",
+         "\nYou could try setting src = \"yahoo\" instead.",
+         "\nSee help(\"Defunct\") and help(\"quantmod-defunct\")")
+  .Defunct("getSymbols", "quantmod", msg = msg)
 }
 # }}}
 
@@ -689,7 +626,10 @@ function(Symbols,env,return.class='xts',
                 warning(paste('could not load symbol(s): ',paste(missing.db.symbol,collapse=', ')))
                 Symbols <- Symbols[Symbols %in% db.Symbols]
         }
-        for(i in 1:length(Symbols)) {
+        returnSym <- Symbols
+        noDataSym <- NULL
+        for(i in seq_along(Symbols)) {
+            test <- try({
             if(verbose) {
                 cat(paste('Loading ',Symbols[[i]],
                     paste(rep('.',10-nchar(Symbols[[i]])),collapse=''),
@@ -716,10 +656,19 @@ function(Symbols,env,return.class='xts',
             if(auto.assign)
               assign(Symbols[[i]],fr,env)
             if(verbose) cat('done\n')
+            }, silent = TRUE)
+            if (inherits(test, "try-error")) {
+                msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                              ".\n", attr(test, "condition")$message)
+                if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+                  stop(msg)
+                }
+                warning(msg, call. = FALSE, immediate. = TRUE)
+            }
         }
         DBI::dbDisconnect(con)
         if(auto.assign)
-          return(Symbols)
+          return(setdiff(returnSym, noDataSym))
         return(fr)
 }
 "getSymbols.sqlite" <- getSymbols.SQLite
@@ -758,7 +707,10 @@ function(Symbols,env,return.class='xts',
                 warning(paste('could not load symbol(s): ',paste(missing.db.symbol,collapse=', ')))
                 Symbols <- Symbols[Symbols %in% db.Symbols]
         }
-        for(i in 1:length(Symbols)) {
+        returnSym <- Symbols
+        noDataSym <- NULL
+        for(i in seq_along(Symbols)) {
+            test <- try({
             if(verbose) {
                 cat(paste('Loading ',Symbols[[i]],paste(rep('.',10-nchar(Symbols[[i]])),collapse=''),sep=''))
             }
@@ -776,10 +728,20 @@ function(Symbols,env,return.class='xts',
             if(auto.assign)
               assign(Symbols[[i]],fr,env)
             if(verbose) cat('done\n')
+            }, silent = TRUE)
+            if (inherits(test, "try-error")) {
+              msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                            ".\n", attr(test, "condition")$message)
+              if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+                stop(msg)
+              }
+              warning(msg, call. = FALSE, immediate. = TRUE)
+              noDataSym <- c(noDataSym, returnSym[[i]])
+            }
         }
         DBI::dbDisconnect(con)
         if(auto.assign)
-          return(Symbols)
+          return(setdiff(returnSym, noDataSym))
         return(fr)
 }
 "getSymbols.mysql" <- getSymbols.MySQL
@@ -798,13 +760,15 @@ function(Symbols,env,return.class='xts',
      if(!hasArg("auto.assign")) auto.assign <- TRUE
      FRED.URL <- "https://fred.stlouisfed.org/series"
 
-     tmp <- tempfile()
-     on.exit(unlink(tmp))
-     for(i in 1:length(Symbols)) {
+     returnSym <- Symbols
+     noDataSym <- NULL
+
+     for(i in seq_along(Symbols)) {
        if(verbose) cat("downloading ",Symbols[[i]],".....\n\n")
+       test <- try({
        URL <- paste(FRED.URL, "/", Symbols[[i]], "/downloaddata/", Symbols[[i]], ".csv", sep="")
-       try.download.file(URL, destfile=tmp, quiet=!verbose, ...)
-       fr <- read.csv(tmp,na.string=".")
+       fr <- read.csv(curl::curl(URL),na.string=".")
+
        if(verbose) cat("done.\n")
        fr <- xts(as.matrix(fr[,-1]),
                  as.Date(fr[,1],origin='1970-01-01'),
@@ -815,9 +779,19 @@ function(Symbols,env,return.class='xts',
        Symbols[[i]] <-toupper(gsub('\\^','',Symbols[[i]])) 
        if(auto.assign)
          assign(Symbols[[i]],fr,env)
+       }, silent = TRUE)
+       if (inherits(test, "try-error")) {
+         msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                       ".\n", attr(test, "condition")$message)
+         if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+           stop(msg)
+         }
+         warning(msg, call. = FALSE, immediate. = TRUE)
+         noDataSym <- c(noDataSym, returnSym[[i]])
+       }
      }
      if(auto.assign)
-       return(Symbols)
+       return(setdiff(returnSym, noDataSym))
      return(fr)
 } #}}}
 
@@ -903,7 +877,11 @@ function(Symbols,env,
   if(!hasArg("verbose")) verbose <- FALSE
   if(!hasArg("auto.assign")) auto.assign <- TRUE
 
-  for(i in 1:length(Symbols)) {
+  returnSym <- Symbols
+  noDataSym <- NULL
+
+  for(i in seq_along(Symbols)) {
+    test <- try({
     return.class <- getSymbolLookup()[[Symbols[[i]]]]$return.class
     return.class <- ifelse(is.null(return.class),default.return.class,
                            return.class)
@@ -944,9 +922,19 @@ function(Symbols,env,
     Symbols[[i]] <-toupper(gsub('\\^','',Symbols[[i]])) 
     if(auto.assign)
       assign(Symbols[[i]],fr,env)
+    }, silent = TRUE)
+    if (inherits(test, "try-error")) {
+       msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                     ".\n", attr(test, "condition")$message)
+       if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+         stop(msg)
+       }
+       warning(msg, call. = FALSE, immediate. = TRUE)
+      noDataSym <- c(noDataSym, returnSym[[i]])
+    }
     }
     if(auto.assign)
-      return(Symbols)
+      return(setdiff(returnSym, noDataSym))
     return(fr)
 }
 #}}}
@@ -972,7 +960,11 @@ function(Symbols,env,
   if(!hasArg("verbose")) verbose <- FALSE
   if(!hasArg("auto.assign")) auto.assign <- TRUE
 
-  for(i in 1:length(Symbols)) {
+  returnSym <- Symbols
+  noDataSym <- NULL
+
+  for(i in seq_along(Symbols)) {
+    test <- try({
     return.class <- getSymbolLookup()[[Symbols[[i]]]]$return.class
     return.class <- ifelse(is.null(return.class),default.return.class,
                            return.class)
@@ -1003,9 +995,19 @@ function(Symbols,env,
     Symbols[[i]] <-toupper(gsub('\\^','',Symbols[[i]])) 
     if(auto.assign)
       assign(Symbols[[i]],fr,env)
+    }, silent = TRUE)
+    if (inherits(test, "try-error")) {
+      msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                    ".\n", attr(test, "condition")$message)
+      if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+        stop(msg)
+      }
+      warning(msg, call. = FALSE, immediate. = TRUE)
+      noDataSym <- c(noDataSym, returnSym[[i]])
+    }
     }
     if(auto.assign)
-      return(Symbols)
+      return(setdiff(returnSym, noDataSym))
     return(fr)
 }
 #}}}
@@ -1031,7 +1033,11 @@ function(Symbols,env,
   if(!hasArg("verbose")) verbose <- FALSE
   if(!hasArg("auto.assign")) auto.assign <- TRUE
 
-  for(i in 1:length(Symbols)) {
+  returnSym <- Symbols
+  noDataSym <- NULL
+
+  for(i in seq_along(Symbols)) {
+    test <- try({
     return.class <- getSymbolLookup()[[Symbols[[i]]]]$return.class
     return.class <- ifelse(is.null(return.class),default.return.class,
                            return.class)
@@ -1063,9 +1069,19 @@ function(Symbols,env,
     Symbols[[i]] <-toupper(gsub('\\^','',Symbols[[i]])) 
     if(auto.assign)
       assign(Symbols[[i]],fr,env)
+    }, silent = TRUE)
+    if (inherits(test, "try-error")) {
+      msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                    ".\n", attr(test, "condition")$message)
+      if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+        stop(msg)
+      }
+      warning(msg, call. = FALSE, immediate. = TRUE)
+      noDataSym <- c(noDataSym, returnSym[[i]])
+    }
     }
     if(auto.assign)
-      return(Symbols)
+      return(setdiff(returnSym, noDataSym))
     return(fr)
 }
 #}}}
@@ -1094,7 +1110,11 @@ useRTH = '1', whatToShow = 'TRADES', time.format = '1', ...)
   
     if(missing(endDateTime)) endDateTime <- NULL
   
-    for(i in 1:length(Symbols)) {
+    returnSym <- Symbols
+    noDataSym <- NULL
+
+    for(i in seq_along(Symbols)) {
+      test <- try({
       Contract <- getSymbolLookup()[[Symbols[i]]]$Contract
       if(inherits(Contract,'twsContract')) {
         fr <- do.call('reqHistoricalData',list(tws, Contract, endDateTime=endDateTime,
@@ -1115,9 +1135,19 @@ useRTH = '1', whatToShow = 'TRADES', time.format = '1', ...)
       } else {
         warning(paste('unable to load',Symbols[i],': missing twsContract definition'))
       }
+    }, silent = TRUE)
+    if (inherits(test, "try-error")) {
+      msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                    ".\n", attr(test, "condition")$message)
+      if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+        stop(msg)
+      }
+      warning(msg, call. = FALSE, immediate. = TRUE)
+      noDataSym <- c(noDataSym, returnSym[[i]])
+    }
     }
     if(auto.assign)
-      return(Symbols)
+      return(setdiff(returnSym, noDataSym))
     return(fr) 
   }
 }
@@ -1172,9 +1202,11 @@ function(Symbols,env,return.class='xts',
      if(!hasArg("verbose")) verbose <- FALSE
      if(!hasArg("auto.assign")) auto.assign <- TRUE
 
-     tmp <- tempfile()
-     on.exit(unlink(tmp))
-     for(i in 1:length(Symbols)) {
+     returnSym <- Symbols
+     noDataSym <- NULL
+
+     for(i in seq_along(Symbols)) {
+       test <- try({
        return.class <- getSymbolLookup()[[Symbols[[i]]]]$return.class
        return.class <- ifelse(is.null(return.class),default.return.class,
                               return.class)
@@ -1200,16 +1232,15 @@ function(Symbols,env,return.class='xts',
                    " Symbol: ", Symbols[[i]])
        }
        oanda.URL <- paste0("https://www.oanda.com/fx-for-business/",
-                           "historical-rates/api/update/?&widget=1",
-                           "&source=OANDA&display=absolute&adjustment=0",
-                           "&data_range=c",
-                           "&quote_currency=", currency.pair[1],
+                           "historical-rates/api/data/update/",
+                           "?&source=OANDA&adjustment=0",
+                           "&base_currency=", currency.pair[1],
                            "&start_date=", from,
                            "&end_date=", to,
                            "&period=daily",
                            "&price=mid",
                            "&view=table",
-                           "&base_currency_0=", currency.pair[2])
+                           "&quote_currency_0=", currency.pair[2])
        # Fetch data (jsonlite::fromJSON will handle connection)
        tbl <- jsonlite::fromJSON(oanda.URL, simplifyVector = FALSE)
        Data <- tbl[[1]][[1]]$data
@@ -1231,9 +1262,19 @@ function(Symbols,env,return.class='xts',
        Symbols[[i]] <-toupper(gsub('\\^|/','',Symbols[[i]])) 
        if(auto.assign)
          assign(Symbols[[i]],fr,env)
+     }, silent = TRUE)
+     if (inherits(test, "try-error")) {
+       msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                     ".\n", attr(test, "condition")$message)
+       if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+         stop(msg)
+       }
+       warning(msg, call. = FALSE, immediate. = TRUE)
+       noDataSym <- c(noDataSym, returnSym[[i]])
+     }
      }
      if(auto.assign)
-       return(Symbols)
+       return(setdiff(returnSym, noDataSym))
      return(fr)
 }#}}}
 
@@ -1278,9 +1319,6 @@ getSymbols.av <- function(Symbols, env, api.key,
          " cannot be loaded.", call.=FALSE)
   }
   
-  tmp <- tempfile()
-  on.exit(file.remove(tmp))
-  
   #
   # For daily, weekly, and monthly data, timestamps are "yyyy-mm-dd".
   # For intraday data, timestamps are "yyyy-mm-dd HH:MM:SS".
@@ -1301,22 +1339,22 @@ getSymbols.av <- function(Symbols, env, api.key,
     periodicity <- if (is.null(periodicity)) default.periodicity else periodicity
     periodicity <- match.arg(periodicity, valid.periodicity)
     
-    if (adjusted && periodicity != "daily")
-      stop("getSymbols.av: Only daily data can be adjusted.", call.=FALSE)
+    if (adjusted && periodicity == "intraday")
+      stop("getSymbols.av: Intraday data cannot be adjusted.", call.=FALSE)
     
     sym.name <- getSymbolLookup()[[sym]]$name
     sym.name <- if (is.null(sym.name)) sym else sym.name
     
-    FUNCTION <-
+    FUNCTION <- paste0("TIME_SERIES_",
       switch(periodicity,
-             daily = if (adjusted) "TIME_SERIES_DAILY_ADJUSTED" else "TIME_SERIES_DAILY",
-             weekly = "TIME_SERIES_WEEKLY",
-             monthly = "TIME_SERIES_MONTHLY",
-             intraday = "TIME_SERIES_INTRADAY" )
+             daily = if (adjusted) "DAILY_ADJUSTED" else "DAILY",
+             weekly = if (adjusted) "WEEKLY_ADJUSTED" else "WEEKLY",
+             monthly = if (adjusted) "MONTHLY_ADJUSTED" else "MONTHLY",
+             intraday = "INTRADAY" ))
     
     if (verbose) cat("loading", sym.name, ".....")
     
-    URL <- paste0("http://www.alphavantage.co/query",
+    URL <- paste0("https://www.alphavantage.co/query",
                   "?function=", FUNCTION,
                   "&symbol=", sym.name,
                   "&interval=", interval,
@@ -1324,10 +1362,8 @@ getSymbols.av <- function(Symbols, env, api.key,
                   "&datatype=", data.type,
                   "&apikey=", api.key)
     
-    download.file(url=URL, destfile=tmp, quiet=!verbose)
-
     if (data.type == "json") {
-      lst <- jsonlite::fromJSON(tmp)
+      lst <- jsonlite::fromJSON(URL)
 
       #
       # Errors return a list with one element: An error message
@@ -1400,7 +1436,7 @@ getSymbols.av <- function(Symbols, env, api.key,
       mat <- xts(rows, tm.stamps, src="alphavantage", updated=updated)
       mat <- convert.time.series(mat, return.class=return.class)
     } else {
-      mat <- as.xts(read.zoo(tmp, header=TRUE, sep=","),
+      mat <- as.xts(read.zoo(curl::curl(URL), header=TRUE, sep=","),
                     src="alphavantage", updated=Sys.time())
       # convert column names to symbol.series
       cn <- colnames(mat)
@@ -1414,12 +1450,29 @@ getSymbols.av <- function(Symbols, env, api.key,
     return(mat)
   }
   
-  matrices <- lapply(Symbols, FUN=downloadOne,
-                     default.return.class=default.return.class,
-                     default.periodicity=default.periodicity)
-  
+  returnSym <- Symbols
+  noDataSym <- NULL
+  matrices <- list()
+
+  for(i in seq_along(Symbols)) {
+    test <- try({
+      matrices[[i]] <- downloadOne(Symbols[[i]],
+        default.return.class = default.return.class,
+        default.periodicity = default.periodicity)
+    }, silent = TRUE)
+    if (inherits(test, "try-error")) {
+      msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                    ".\n", attr(test, "condition")$message)
+      if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+        stop(msg)
+      }
+      warning(msg, call. = FALSE, immediate. = TRUE)
+      noDataSym <- c(noDataSym, returnSym[[i]])
+    }
+  }
+
   if (auto.assign) {
-    return(Symbols)
+    return(setdiff(returnSym, noDataSym))
   } else {
     return(matrices[[1]])
   }
@@ -1428,6 +1481,147 @@ getSymbols.av <- function(Symbols, env, api.key,
 # Mnemonic alias, letting callers use getSymbols("IBM", src="alphavantage")
 getSymbols.alphavantage <- getSymbols.av
 
+#
+#  Download OHLC Data From Tiingo
+#  
+#  Meant to be called internally by getSymbols().
+#  
+getSymbols.tiingo <- function(Symbols, env, api.key,
+                              return.class="xts",
+                              periodicity="daily",
+                              adjust=FALSE,
+                              from='2007-01-01',
+                              to=Sys.Date(),
+                              data.type="json",
+                              ...) {
+  
+  importDefaults("getSymbols.tiingo")
+  this.env <- environment()
+  for (var in names(list(...))) {
+    assign(var, list(...)[[var]], this.env)
+  }
+  
+  if (!hasArg("api.key")) {
+    stop("getSymbols.tiingo: An API key is required (api.key). Register",
+         " at https://api.tiingo.com.", call.=FALSE)
+  }
+  if (!hasArg("auto.assign")) auto.assign <- TRUE
+  if (!hasArg("verbose")) verbose <- FALSE
+  if (!hasArg("warnings")) warnings <- TRUE
+  
+  valid.periodicity <- c("daily", "weekly", "monthly", "annually")
+  periodicity <- match.arg(periodicity, valid.periodicity)
+  default.return.class <- return.class
+  default.periodicity <- periodicity
+  
+  if (!requireNamespace("jsonlite", quietly=TRUE)) {
+    stop("getSymbols.tiingo: Package", dQuote("jsonlite"), "is required but",
+         " cannot be loaded.", call.=FALSE)
+  }
+  
+  downloadOne <- function(sym, default.return.class, default.periodicity) {
+    
+    return.class <- getSymbolLookup()[[sym]]$return.class
+    return.class <- if (is.null(return.class)) default.return.class else return.class
+    periodicity <- getSymbolLookup()[[sym]]$periodicity
+    periodicity <- if (is.null(periodicity)) default.periodicity else periodicity
+    periodicity <- match.arg(periodicity, valid.periodicity)
+    sym.name <- getSymbolLookup()[[sym]]$name
+    sym.name <- if (is.null(sym.name)) sym else sym.name
+    
+    if (verbose) cat("loading", sym.name, ".....")
+    from.strftime <- strftime(from, format = "%Y-%m-%d")
+    to.strftime <- strftime(to, format = "%Y-%m-%d")
+    
+    tiingo.names <- c("open", "high", "low", "close", "volume",
+                      "adjOpen", "adjHigh", "adjLow", "adjClose",
+                      "adjVolume", "divCash", "splitFactor")
+    qm.names <- paste(sym, c("Open", "High", "Low", "Close", "Volume",
+                             "Open", "High", "Low", "Close", "Volume",
+                             "DivCash", "SplitFactor"), sep=".")
+    if (isTRUE(adjust)) {
+      return.columns <- tiingo.names[6:10]
+    } else {
+      return.columns <- tiingo.names[1:5]
+    }
+    URL <- paste0("https://api.tiingo.com/tiingo/",
+                  periodicity, "/",
+                  sym.name, "/prices",
+                  "?startDate=", from.strftime,
+                  "&endDate=", to.strftime,
+                  "&format=", data.type,
+                  "&token=", api.key,
+                  "&columns=", paste0(return.columns, collapse=","))
+    # If rate limit is hit, the csv API returns HTTP 200 (OK), while json API
+    # returns HTTP 429. The latter caused download.file() to error, but the
+    # contents of 'tmp' still contain the error message.
+
+    if (data.type == "json") {
+      stock.data <- jsonlite::fromJSON(URL)
+      if (verbose) cat("done.\n")
+    } else {
+      stock.data <- read.csv(curl::curl(URL), as.is=TRUE)
+    }
+    # check for error
+    if (!all(return.columns %in% names(stock.data))) {
+      if (data.type == "json") {
+        msg <- stock.data$detail
+      } else {
+        msg <- readLines(curl::curl(URL), warn=FALSE)
+      }
+      msg <- sub("Error: ", "", msg)
+      stop(msg, call. = FALSE)
+    }
+    tm.stamps <- as.POSIXct(stock.data[, "date"], ...)
+    stock.data[, "date"] <- NULL
+
+    # adjusted column names
+    adjcols <- grepl("^adj", colnames(stock.data))
+    # order Tiingo column names before converting to quantmod names
+    stock.data <- OHLCV(stock.data)
+    if (any(adjcols)) {
+      # put adjusted columns last
+      stock.data <- stock.data[, c(which(!adjcols), which(adjcols))]
+    }
+    # now convert to quantmod column names
+    colnames(stock.data) <- qm.names[match(colnames(stock.data), tiingo.names)]
+
+    # convert data to xts
+    xts.data <- xts(stock.data, tm.stamps, src="tiingo", updated=Sys.time())
+    xts.data <- convert.time.series(xts.data, return.class=return.class)
+    if (auto.assign)
+      assign(sym, xts.data, env)
+    return(xts.data)
+  }
+  
+  returnSym <- Symbols
+  noDataSym <- NULL
+  matrices <- list()
+
+  for(i in seq_along(Symbols)) {
+    test <- try({
+      matrices[[i]] <- downloadOne(Symbols[[i]],
+        default.return.class = default.return.class,
+        default.periodicity = default.periodicity)
+    }, silent = TRUE)
+    if (inherits(test, "try-error")) {
+      msg <- paste0("Unable to import ", dQuote(returnSym[[i]]),
+                    ".\n", attr(test, "condition")$message)
+      if (hasArg(".has1sym.") && match.call(expand.dots=TRUE)$.has1sym.) {
+        stop(msg)
+      }
+      warning(msg, call. = FALSE, immediate. = TRUE)
+      noDataSym <- c(noDataSym, returnSym[[i]])
+    }
+  }
+  
+  if (auto.assign) {
+    return(setdiff(returnSym, noDataSym))
+  } else {
+    return(matrices[[1]])
+  }
+}
+
 # convert.time.series {{{
 `convert.time.series` <- function(fr,return.class) {
        if('quantmod.OHLC' %in% return.class) {
